@@ -46,6 +46,7 @@
 #include "gr-unit.h"
 #include "gr-chef-dialog.h"
 #include "gr-cooking-view.h"
+#include "gr-window.h"
 
 #ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
@@ -1125,10 +1126,12 @@ static void update_author_label (GrEditPage *page,
 static void
 chef_done (GrChefDialog *dialog, GrChef *chef, GrEditPage *page)
 {
-        g_free (page->author);
-        page->author = g_strdup (gr_chef_get_id (chef));
-
-        update_author_label (page, chef);
+        if (chef) {
+                g_free (page->author);
+                page->author = g_strdup (gr_chef_get_id (chef));
+                update_author_label (page, chef);
+        }
+        gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
 static gboolean
@@ -1147,12 +1150,11 @@ edit_chef (GrEditPage *page)
         author = page->author;
         chef = gr_recipe_store_get_chef (store, author ? author : "");
 
-        dialog = gr_chef_dialog_new (GTK_WINDOW (win), chef);
-        gr_chef_dialog_can_create (dialog, TRUE);
+        dialog = gr_chef_dialog_new (chef, TRUE);
         g_signal_connect (dialog, "done", G_CALLBACK (chef_done), page);
 
         gtk_window_set_title (GTK_WINDOW (dialog), _("Recipe Author"));
-        gtk_window_present (GTK_WINDOW (dialog));
+        gr_window_present_dialog (GR_WINDOW (win), GTK_WINDOW (dialog));
 
         return TRUE;
 }
@@ -1697,7 +1699,7 @@ gr_edit_page_clear (GrEditPage *page)
 
         store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
 
-        gtk_label_set_label (GTK_LABEL (page->name_label), _("Name your recipe"));
+        gtk_label_set_label (GTK_LABEL (page->name_label), _("Name Your Recipe"));
         gtk_entry_set_text (GTK_ENTRY (page->name_entry), "");
         set_combo_value (GTK_COMBO_BOX (page->cuisine_combo), "");
         set_combo_value (GTK_COMBO_BOX (page->category_combo), "");
