@@ -20,11 +20,16 @@
 
 #include "config.h"
 
+#include <sys/stat.h>
+#include <sys/types.h>
+
 #include <glib/gi18n.h>
+#include <gio/gunixoutputstream.h>
 
 #include "gr-image-viewer.h"
 #include "gr-images.h"
 #include "gr-utils.h"
+#include "gr-window.h"
 
 
 struct _GrImageViewer
@@ -55,9 +60,11 @@ G_DEFINE_TYPE (GrImageViewer, gr_image_viewer, GTK_TYPE_BOX)
 enum {
         PROP_0,
         PROP_IMAGES,
+        PROP_INDEX,
         N_PROPS
 };
 
+static guint activate_signal;
 
 GrImageViewer *
 gr_image_viewer_new (void)
@@ -118,6 +125,8 @@ set_current_image (GrImageViewer *viewer)
         child = gtk_flow_box_get_child_at_index (GTK_FLOW_BOX (viewer->preview_list), viewer->index);
         if (child)
                 gtk_flow_box_select_child (GTK_FLOW_BOX (viewer->preview_list), child);
+
+        g_object_notify (G_OBJECT (viewer), "index");
 }
 
 static void
@@ -252,10 +261,23 @@ motion_notify (GtkWidget     *widget,
 }
 
 static void
-button_press (GrImageViewer *viewer)
+button_press (GtkGesture *gesture,
+              int         n_press,
+              double      x,
+              double      y,
+              GrImageViewer *viewer)
 {
-        toggle_preview (viewer);
-        gtk_widget_grab_focus (viewer->event_box);
+        int button;
+
+        button = gtk_gesture_single_get_current_button (GTK_GESTURE_SINGLE (gesture));
+
+        if (button == GDK_BUTTON_PRIMARY) {
+                g_signal_emit (viewer, activate_signal, 0);
+        }
+        else {
+                toggle_preview (viewer);
+                gtk_widget_grab_focus (viewer->event_box);
+        }
 }
 
 static void
@@ -290,6 +312,57 @@ preview_selected (GrImageViewer *viewer)
         }
 }
 
+static void
+add_image (GrImageViewer  *viewer,
+           GrRotatedImage *ri,
+           gboolean        select)
+{
+        g_array_append_vals (viewer->images, ri, 1);
+        ri = &g_array_index (viewer->images, GrRotatedImage, viewer->images->len - 1);
+        ri->path = g_strdup (ri->path);
+
+        populate_preview (viewer);
+        if (select)
+                viewer->index = viewer->images->len - 1;
+        set_current_image (viewer);
+
+        g_object_notify (G_OBJECT (viewer), "images");
+}
+
+static void
+image_received (GtkClipboard *clipboard,
+                GdkPixbuf    *pixbuf,
+                gpointer      data)
+{
+        GrImageViewer *viewer = data;
+
+        if (pixbuf) {
+                GrRotatedImage ri;
+                g_autofree char *dir = NULL;
+                g_autofree char *path = NULL;
+                int fd;
+                g_autoptr(GOutputStream) stream = NULL;
+                g_autoptr(GError) error = NULL;
+
+                dir = g_build_filename (g_get_user_data_dir (), "recipes", NULL);
+                g_mkdir_with_parents (dir, S_IRWXU | S_IRWXG | S_IRWXO);
+
+                path = g_build_filename (dir, "clipboardXXXXXX.png", NULL);
+                fd = g_mkstemp (path);
+                stream = g_unix_output_stream_new (fd, TRUE);
+
+                if (!gdk_pixbuf_save_to_stream (pixbuf, stream, "png", NULL, &error, NULL)) {
+                        g_message ("Saving clipboard contents failed: %s", error->message);
+                        return;
+                }
+
+                ri.path = g_strdup (path);
+                ri.angle = 0;
+
+                add_image (viewer, &ri, TRUE);
+        }
+}
+
 static gboolean
 key_press_event (GtkWidget     *widget,
                  GdkEvent      *event,
@@ -313,6 +386,13 @@ key_press_event (GtkWidget     *widget,
                 next_image (viewer);
                 return TRUE;
         }
+        else if (key->keyval == GDK_KEY_v && (key->state & GDK_CONTROL_MASK) != 0) {
+                GtkClipboard *clipboard;
+
+                clipboard = gtk_widget_get_clipboard (GTK_WIDGET (viewer), GDK_SELECTION_CLIPBOARD);
+                gtk_clipboard_request_image (clipboard, image_received, viewer);
+                return TRUE;
+        }
 
         return FALSE;
 }
@@ -330,8 +410,9 @@ gr_image_viewer_init (GrImageViewer *self)
         g_signal_connect (self->event_box, "key-press-event", G_CALLBACK (key_press_event), self);
 
         self->gesture = gtk_gesture_multi_press_new (self->event_box);
+        gtk_gesture_single_set_button (GTK_GESTURE_SINGLE (self->gesture), 0);
         gtk_event_controller_set_propagation_phase (GTK_EVENT_CONTROLLER (self->gesture), GTK_PHASE_BUBBLE);
-        g_signal_connect_swapped (self->gesture, "pressed", G_CALLBACK (button_press), self);
+        g_signal_connect (self->gesture, "pressed", G_CALLBACK (button_press), self);
         self->images = gr_rotated_image_array_new ();
 }
 
@@ -349,6 +430,10 @@ gr_image_viewer_get_property (GObject    *object,
                   g_value_set_boxed (value, self->images);
                   break;
 
+          case PROP_INDEX:
+                  g_value_set_int (value, self->index);
+                  break;
+
           default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
           }
@@ -365,7 +450,13 @@ gr_image_viewer_set_property (GObject      *object,
         switch (prop_id)
           {
           case PROP_IMAGES:
-                  gr_image_viewer_set_images (self, (GArray *) g_value_get_boxed (value));
+                  gr_image_viewer_set_images (self,
+                                              (GArray *) g_value_get_boxed (value),
+                                              gr_image_viewer_get_index (self));
+                  break;
+
+          case PROP_INDEX:
+                  gr_image_viewer_show_image (self, g_value_get_int (value));
                   break;
 
           default:
@@ -389,6 +480,19 @@ gr_image_viewer_class_init (GrImageViewerClass *klass)
                                     G_PARAM_READWRITE);
         g_object_class_install_property (object_class, PROP_IMAGES, pspec);
 
+        pspec = g_param_spec_int ("index", NULL, NULL,
+                                  0, G_MAXINT, 0,
+                                  G_PARAM_READWRITE);
+        g_object_class_install_property (object_class, PROP_INDEX, pspec);
+
+        activate_signal = g_signal_new ("activate",
+                                        G_TYPE_FROM_CLASS (object_class),
+                                        G_SIGNAL_RUN_LAST,
+                                        0,
+                                        NULL, NULL,
+                                        NULL,
+                                        G_TYPE_NONE, 0);
+
         gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Recipes/gr-image-viewer.ui");
         gtk_widget_class_bind_template_child (widget_class, GrImageViewer, image1);
         gtk_widget_class_bind_template_child (widget_class, GrImageViewer, image2);
@@ -404,26 +508,10 @@ gr_image_viewer_class_init (GrImageViewerClass *klass)
         gtk_widget_class_bind_template_callback (widget_class, preview_selected);
 }
 
-static void
-add_image (GrImageViewer  *viewer,
-           GrRotatedImage *ri,
-           gboolean        select)
-{
-        g_array_append_vals (viewer->images, ri, 1);
-        ri = &g_array_index (viewer->images, GrRotatedImage, viewer->images->len - 1);
-        ri->path = g_strdup (ri->path);
-
-        populate_preview (viewer);
-        if (select)
-                viewer->index = viewer->images->len - 1;
-        set_current_image (viewer);
-
-        g_object_notify (G_OBJECT (viewer), "images");
-}
-
 void
 gr_image_viewer_set_images (GrImageViewer *viewer,
-                            GArray        *images)
+                            GArray        *images,
+                            int            index)
 {
         int i;
 
@@ -438,7 +526,7 @@ gr_image_viewer_set_images (GrImageViewer *viewer,
         }
 
         populate_preview (viewer);
-        viewer->index = 0;
+        viewer->index = index;
         set_current_image (viewer);
         hide_controls (viewer);
 
@@ -451,19 +539,20 @@ file_chooser_response (GtkNativeDialog *self,
                        GrImageViewer   *viewer)
 {
         if (response_id == GTK_RESPONSE_ACCEPT) {
-                GrRotatedImage ri;
-                const char *dark;
+                GSList *names, *l;
 
-                ri.path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (self));
-                ri.angle = 0;
+                names = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER (self));
+                for (l = names; l; l = l->next) {
+                        GrRotatedImage ri;
 
-                dark = gtk_file_chooser_get_choice (GTK_FILE_CHOOSER (self), "dark");
+                        ri.path = g_strdup (l->data);
+                        ri.angle = 0;
 
-                ri.dark_text = g_strcmp0 (dark, "true") == 0;
+                        add_image (viewer, &ri, TRUE);
 
-                add_image (viewer, &ri, TRUE);
-
-                g_free (ri.path);
+                        g_free (ri.path);
+                }
+                g_slist_free_full (names, g_free);
 
                 show_controls (viewer);
         }
@@ -503,7 +592,7 @@ open_filechooser (GrImageViewer *viewer)
         gtk_file_filter_add_mime_type (filter, "image/*");
         gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), filter);
         gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (chooser), filter);
-        gtk_file_chooser_add_choice (GTK_FILE_CHOOSER (chooser), "dark", _("Use dark text"), NULL, NULL);
+        gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (chooser), TRUE);
 
         g_signal_connect (chooser, "response", G_CALLBACK (file_chooser_response), viewer);
 
@@ -562,6 +651,14 @@ void
 gr_image_viewer_show_image (GrImageViewer *viewer,
                             int            idx)
 {
-        viewer->index = idx % viewer->images->len;
-        set_current_image (viewer);
+        if (viewer->images->len > 0) {
+                viewer->index = idx % viewer->images->len;
+                set_current_image (viewer);
+        }
+}
+
+int
+gr_image_viewer_get_index (GrImageViewer *viewer)
+{
+        return viewer->images->len;
 }
