@@ -228,17 +228,17 @@ get_category_title (GrDiets diet)
 {
         switch (diet) {
         case GR_DIET_GLUTEN_FREE:
-                return _("Gluten-free recipes");
+                return _("Gluten-free Recipes");
         case GR_DIET_NUT_FREE:
-                return _("Nut-free recipes");
+                return _("Nut-free Recipes");
         case GR_DIET_VEGAN:
-                return _("Vegan recipes");
+                return _("Vegan Recipes");
         case GR_DIET_VEGETARIAN:
-                return _("Vegetarian recipes");
+                return _("Vegetarian Recipes");
         case GR_DIET_MILK_FREE:
-                return _("Milk-free recipes");
+                return _("Milk-free Recipes");
         default:
-                return  _("Other dietary restrictions");
+                return  _("Other Dietary Restrictions");
         }
 }
 
@@ -442,6 +442,13 @@ gr_list_page_repopulate (GrListPage *page)
                 gr_list_page_populate_from_list (page, page->recipes);
 }
 
+static void
+repopulate (GrListPage *page)
+{
+        if (gtk_widget_is_drawable (GTK_WIDGET (page)))
+                gr_list_page_repopulate (page);
+}
+
 static void
 connect_store_signals (GrListPage *page)
 {
@@ -449,10 +456,9 @@ connect_store_signals (GrListPage *page)
 
         store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
 
-        /* FIXME: inefficient */
-        g_signal_connect_swapped (store, "recipe-added", G_CALLBACK (gr_list_page_repopulate), page);
-        g_signal_connect_swapped (store, "recipe-removed", G_CALLBACK (gr_list_page_repopulate), page);
-        g_signal_connect_swapped (store, "recipe-changed", G_CALLBACK (gr_list_page_repopulate), page);
+        g_signal_connect_swapped (store, "recipe-added", G_CALLBACK (repopulate), page);
+        g_signal_connect_swapped (store, "recipe-removed", G_CALLBACK (repopulate), page);
+        g_signal_connect_swapped (store, "recipe-changed", G_CALLBACK (repopulate), page);
 }
 
 void
