@@ -29,12 +29,14 @@
 #endif
 
 #include "gr-recipe-exporter.h"
+#include "gr-recipe-formatter.h"
 #include "gr-images.h"
 #include "gr-chef.h"
 #include "gr-recipe.h"
 #include "gr-recipe-store.h"
 #include "gr-app.h"
 #include "gr-utils.h"
+#include "gr-mail.h"
 
 
 struct _GrRecipeExporter
@@ -53,7 +55,11 @@ struct _GrRecipeExporter
         GList *sources;
         char *dir;
 
+        gboolean contribute;
+
         GtkWidget *dialog_heading;
+        GtkWidget *friend_button;
+        GtkWidget *contribute_button;
 };
 
 G_DEFINE_TYPE (GrRecipeExporter, gr_recipe_exporter, G_TYPE_OBJECT)
@@ -123,26 +129,72 @@ static void
 completed_cb (AutoarCompressor *compressor,
               GrRecipeExporter *exporter)
 {
-        GtkWidget *dialog;
-        g_autofree char *path =  NULL;
-        int n;
+        g_autofree char *path = NULL;
+        g_autoptr(GError) error = NULL;
+        const char *address;
+        const char *subject;
+        g_autofree char *body = NULL;
+        GList *attachments;
+
+        if (exporter->contribute) {
+                address = "recipes-list@gnome.org";
+                subject = _("Recipe contribution");
+                body = g_strdup (_("Please accept my attached recipe contribution."));
+        }
+        else {
+                GString *s;
+                GList *l;
+
+                address = "";
+                s = g_string_new ("");
+
+                if (exporter->recipes->next == NULL) {
+                        subject = _("Try this recipe");
+                        g_string_append (s, _("Hi,\n\nyou should try this recipe."));
+                }
+                else {
+                        subject = _("Try these recipes");
+                        g_string_append (s, _("Hi,\n\nyou should try these recipes."));
+                }
+                g_string_append (s, "\n\n");
+                g_string_append (s, _("(The attached file can be imported into GNOME recipes.)"));
+
+                for (l = exporter->recipes; l; l = l->next) {
+                        g_autofree char *formatted = NULL;
+
+                        formatted = gr_recipe_format (GR_RECIPE (l->data));
+                        g_string_append (s, "\n\n");
+                        g_string_append (s, formatted);
+                }
+
+                body = g_string_free (s, FALSE);
+        }
 
-        n = g_list_length (exporter->recipes);
         path = g_file_get_path (exporter->dest);
 
-        g_message (ngettext ("%d recipe has been exported as “%s”",
-                             "%d recipes have been exported as “%s”", n),
-                   n, path);
+        attachments = g_list_append (NULL, path);
 
-        dialog = gtk_message_dialog_new (exporter->window,
-                                         GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
-                                         GTK_MESSAGE_INFO,
-                                         GTK_BUTTONS_OK,
-                                         ngettext ("%d recipe has been exported as “%s”",
-                                                   "%d recipes have been exported as “%s”", n),
-                                         n, path);
-        g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
-        gtk_widget_show (dialog);
+        if (!gr_send_mail (address, subject, body, attachments, &error)) {
+                GtkWidget *error_dialog;
+
+                g_message ("Sharing the exported recipe failed: %s", error->message);
+
+                error_dialog = gtk_message_dialog_new (GTK_WINDOW (exporter->window),
+                                                       GTK_DIALOG_MODAL |
+                                                       GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                       GTK_MESSAGE_ERROR,
+                                                       GTK_BUTTONS_CLOSE,
+                                                       _("Could not share recipes"));
+                gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (error_dialog),
+                                                          _("There was an error sending the exported "
+                                                            "recipes by e-mail.\n"
+                                                            "The exported data can be found at “%s”."),
+                                                          path);
+                g_signal_connect (error_dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
+                gtk_widget_show (error_dialog);
+        }
+
+        g_list_free (attachments);
 
         cleanup_export (exporter);
 }
@@ -435,9 +487,11 @@ export_dialog_response (GtkWidget        *dialog,
 
                 g_message ("exporting %d recipes now", g_list_length (exporter->recipes));
 
+                exporter->contribute = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (exporter->contribute_button));
+
                 for (i = 0; i < 1000; i++) {
                         g_autofree char *tmp;
-                        tmp = g_strdup_printf ("%s/.var/app/org.gnome.Recipes/recipes%d.tar.gz", g_get_home_dir (), i);
+                        tmp = g_strdup_printf ("%s/.var/app/org.gnome.Recipes/recipes%d.gnome-recipes-export", g_get_home_dir (), i);
                         if (!g_file_test (tmp, G_FILE_TEST_EXISTS)) {
                                 path = g_strdup (tmp);
                                 break;
@@ -448,7 +502,7 @@ export_dialog_response (GtkWidget        *dialog,
                         g_autofree char *dir = NULL;
 
                         dir = g_dir_make_tmp ("recipesXXXXXX", NULL);
-                        path = g_build_filename (dir, "recipes.tar.gz", NULL);
+                        path = g_build_filename (dir, "recipes.gnome-recipes-export", NULL);
                 }
 
                 exporter->output = g_file_new_for_path (path);
@@ -467,8 +521,8 @@ update_heading (GrRecipeExporter *exporter)
         int n;
 
         n = g_list_length (exporter->recipes);
-        tmp = g_strdup_printf (ngettext ("%d recipe selected for export",
-                                         "%d recipes selected for export", n), n);
+        tmp = g_strdup_printf (ngettext ("%d recipe selected for sharing",
+                                         "%d recipes selected for sharing", n), n);
         gtk_label_set_label (GTK_LABEL (exporter->dialog_heading), tmp);
 }
 
@@ -483,6 +537,33 @@ update_export_button (GrRecipeExporter *exporter)
     gtk_widget_set_sensitive (exporter->button_now, TRUE);
 }
 
+static void
+update_contribute_button (GrRecipeExporter *exporter)
+{
+        GList *l;
+        gboolean readonly;
+
+        readonly = FALSE;
+        for (l = exporter->recipes; l; l = l->next) {
+                if (gr_recipe_is_readonly (GR_RECIPE (l->data))) {
+                        readonly = TRUE;
+                        break;
+                }
+        }
+
+        if (readonly) {
+                gtk_widget_set_sensitive (exporter->contribute_button, FALSE);
+                gtk_widget_set_tooltip_text (exporter->contribute_button, _("Some of the selected recipes are included\n"
+                                                                            "with GNOME recipes. You should only contribute\n"
+                                                                            "your own recipes."));
+                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (exporter->friend_button), TRUE);
+        }
+        else {
+                gtk_widget_set_sensitive (exporter->contribute_button, TRUE);
+                gtk_widget_set_tooltip_text (exporter->contribute_button, "");
+        }
+}
+
 static void
 row_activated (GtkListBox *list,
                GtkListBoxRow *row,
@@ -506,6 +587,7 @@ row_activated (GtkListBox *list,
 
         update_heading (exporter);
         update_export_button (exporter);
+        update_contribute_button (exporter);
 }
 
 static void
@@ -588,7 +670,12 @@ show_export_dialog (GrRecipeExporter *exporter)
         populate_recipe_list (exporter, list);
 
         exporter->dialog_heading = GTK_WIDGET (gtk_builder_get_object (builder, "heading"));
+        exporter->friend_button = GTK_WIDGET (gtk_builder_get_object (builder, "friend_button"));
+        exporter->contribute_button = GTK_WIDGET (gtk_builder_get_object (builder, "contribute_button"));
+
         update_heading (exporter);
+        update_export_button (exporter);
+        update_contribute_button (exporter);
 
         g_signal_connect (dialog, "response", G_CALLBACK (export_dialog_response), exporter);
         gtk_widget_show (dialog);
