@@ -20,6 +20,7 @@
 
 #include "config.h"
 
+#include <stdlib.h>
 #include <string.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -38,6 +39,7 @@ struct _GrRecipe
         char *author;
         char *description;
         GArray *images;
+        int default_image;
 
         char *cuisine;
         char *season;
@@ -75,6 +77,7 @@ enum {
         PROP_AUTHOR,
         PROP_DESCRIPTION,
         PROP_IMAGES,
+        PROP_DEFAULT_IMAGE,
         PROP_CUISINE,
         PROP_SEASON,
         PROP_CATEGORY,
@@ -153,6 +156,10 @@ gr_recipe_get_property (GObject    *object,
                 g_value_set_boxed (value, self->images);
                 break;
 
+        case PROP_DEFAULT_IMAGE:
+                g_value_set_int (value, self->default_image);
+                break;
+
         case PROP_CATEGORY:
                 g_value_set_string (value, self->category);
                 break;
@@ -287,6 +294,11 @@ gr_recipe_set_property (GObject      *object,
                 update_mtime (self);
                 break;
 
+        case PROP_DEFAULT_IMAGE:
+                self->default_image = g_value_get_int (value);
+                update_mtime (self);
+                break;
+
         case PROP_CATEGORY:
                 g_free (self->category);
                 self->category = g_value_dup_string (value);
@@ -417,6 +429,11 @@ gr_recipe_class_init (GrRecipeClass *klass)
                                      G_PARAM_READWRITE);
         g_object_class_install_property (object_class, PROP_IMAGES, pspec);
 
+        pspec = g_param_spec_int ("default-image", NULL, NULL,
+                                  0, G_MAXINT, 0,
+                                  G_PARAM_READWRITE);
+        g_object_class_install_property (object_class, PROP_DEFAULT_IMAGE, pspec);
+
         pspec = g_param_spec_string ("category", NULL, NULL,
                                      NULL,
                                      G_PARAM_READWRITE);
@@ -613,7 +630,8 @@ gr_recipe_contains_garlic (GrRecipe *recipe)
 gboolean
 gr_recipe_contains_dairy (GrRecipe *recipe)
 {
-        return (recipe->diets & GR_DIET_MILK_FREE) == 0;
+        return (recipe->diets & GR_DIET_MILK_FREE) == 0 &&
+               (recipe->diets & GR_DIET_VEGAN) == 0;
 }
 
 gboolean
@@ -640,6 +658,12 @@ gr_recipe_get_mtime (GrRecipe *recipe)
         return recipe->mtime;
 }
 
+int
+gr_recipe_get_default_image (GrRecipe *recipe)
+{
+        return recipe->default_image;
+}
+
 gboolean
 gr_recipe_is_readonly (GrRecipe *recipe)
 {
@@ -710,9 +734,28 @@ gr_recipe_matches (GrRecipe    *recipe,
                 else if (g_str_has_prefix (terms[i], "na:")) {
                         if (recipe->cf_name && strstr (recipe->cf_name, terms[i] + 3) == NULL)
                                 return FALSE;
+
                         continue;
 
                 }
+                else if (g_str_has_prefix (terms[i], "s+:")) {
+                        int level;
+
+                        level = atoi (terms[i] + 3);
+                        if (recipe->spiciness < level)
+                                return FALSE;
+
+                        continue;
+                }
+                else if (g_str_has_prefix (terms[i], "s-:")) {
+                        int level;
+
+                        level = atoi (terms[i] + 3);
+                        if (recipe->spiciness >= level)
+                                return FALSE;
+
+                        continue;
+                }
 
                 if (recipe->cf_name && strstr (recipe->cf_name, terms[i]) != NULL)
                         continue;
