@@ -93,7 +93,7 @@ hits_added_cb (GrRecipeSearch *search,
         for (l = hits; l; l = l->next) {
                 GrRecipe *recipe = l->data;
 
-                pending_search->results = g_list_prepend (pending_search->results, (gpointer)gr_recipe_get_name (recipe));
+                pending_search->results = g_list_prepend (pending_search->results, (gpointer)gr_recipe_get_id (recipe));
         }
 }
 
@@ -209,14 +209,14 @@ handle_get_result_metas (GrShellSearchProvider2 *skeleton,
                 }
 
                 g_variant_builder_init (&meta, G_VARIANT_TYPE ("a{sv}"));
-                g_variant_builder_add (&meta, "{sv}", "id", g_variant_new_string (gr_recipe_get_name (recipe)));
-                g_variant_builder_add (&meta, "{sv}", "name", g_variant_new_string (gr_recipe_get_name (recipe)));
+                g_variant_builder_add (&meta, "{sv}", "id", g_variant_new_string (gr_recipe_get_id (recipe)));
+                g_variant_builder_add (&meta, "{sv}", "name", g_variant_new_string (gr_recipe_get_translated_name (recipe)));
                 pixbuf = gr_recipe_get_pixbuf (recipe);
                 if (pixbuf != NULL)
                         g_variant_builder_add (&meta, "{sv}", "icon", g_icon_serialize (G_ICON (pixbuf)));
-                g_variant_builder_add (&meta, "{sv}", "description", g_variant_new_string (gr_recipe_get_description (recipe)));
+                g_variant_builder_add (&meta, "{sv}", "description", g_variant_new_string (gr_recipe_get_translated_description (recipe)));
                 meta_variant = g_variant_builder_end (&meta);
-                g_hash_table_insert (self->metas_cache, g_strdup (gr_recipe_get_name (recipe)), g_variant_ref_sink (meta_variant));
+                g_hash_table_insert (self->metas_cache, g_strdup (gr_recipe_get_id (recipe)), g_variant_ref_sink (meta_variant));
 
         }
 
