@@ -34,6 +34,9 @@ GdkPixbuf *load_pixbuf_fill_size (const char *path,
                                   int         width,
                                   int         height);
 
+char *ensure_rotated_image (const char *path,
+                            int         angle);
+
 const char *get_pkg_data_dir  (void);
 const char *get_user_data_dir (void);
 
@@ -59,3 +62,7 @@ void record_step (const char *blurb);
 
 gboolean in_flatpak_sandbox (void);
 gboolean portals_available (void);
+
+void all_headers (GtkListBoxRow *row,
+                  GtkListBoxRow *before,
+                  gpointer       user_data);
