@@ -26,6 +26,7 @@
 
 #include "gr-window.h"
 #include "gr-details-page.h"
+#include "gr-chef-dialog.h"
 #include "gr-edit-page.h"
 #include "gr-list-page.h"
 #include "gr-cuisine-page.h"
@@ -37,6 +38,7 @@
 #include "gr-image-page.h"
 #include "gr-query-editor.h"
 #include "gr-recipe-importer.h"
+#include "gr-about-dialog.h"
 
 
 struct _GrWindow
@@ -47,6 +49,7 @@ struct _GrWindow
         GtkWidget *header_start_stack;
         GtkWidget *header_title_stack;
         GtkWidget *header_end_stack;
+        GtkWidget *back_button;
         GtkWidget *search_button;
         GtkWidget *cooking_button;
         GtkWidget *search_bar;
@@ -69,8 +72,14 @@ struct _GrWindow
         GtkWidget *shopping_added_revealer;
         guint shopping_timeout_id;
 
+        GObject *file_chooser;
         GrRecipeImporter *importer;
 
+        GtkWidget *chef_dialog;
+        GtkWidget *about_dialog;
+
+        GList *dialogs;
+
         GQueue *back_entry_stack;
         gboolean is_fullscreen;
 };
@@ -163,7 +172,7 @@ new_recipe (GrWindow *window)
 
         gr_edit_page_clear (GR_EDIT_PAGE (window->edit_page));
 
-        gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), _("Add a new recipe"));
+        gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), _("Add a New Recipe"));
 
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_start_stack), "back");
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_title_stack), "title");
@@ -259,10 +268,12 @@ visible_page_changed (GrWindow *window)
         }
 
         if (strcmp (visible, "recipes") == 0) {
+                gr_recipes_page_refresh (GR_RECIPES_PAGE (window->recipes_page));
                 gr_recipes_page_unexpand (GR_RECIPES_PAGE (window->recipes_page));
         }
 
         if (strcmp (visible, "cuisines") == 0) {
+                gr_cuisines_page_refresh (GR_CUISINES_PAGE (window->cuisines_page));
                 gr_cuisines_page_unexpand (GR_CUISINES_PAGE (window->cuisines_page));
         }
 }
@@ -341,12 +352,21 @@ window_buttonpress_handler (GtkWidget *widget,
 {
         GrWindow *window = GR_WINDOW (widget);
         const char *visible;
+        GdkEventButton *e = (GdkEventButton *) event;
 
         visible = gtk_stack_get_visible_child_name (GTK_STACK (window->main_stack));
 
         if (strcmp (visible, "cooking") == 0)
           return gr_cooking_page_handle_event (GR_COOKING_PAGE (window->cooking_page), event);
 
+        /* handle mouse back button like a click on our actual back button */
+        if (e->button == 8 &&
+            gtk_widget_can_activate_accel (window->back_button,
+                                           g_signal_lookup ("clicked", GTK_TYPE_BUTTON))) {
+                gr_window_go_back (window);
+                return GDK_EVENT_STOP;
+        }
+
         return GDK_EVENT_PROPAGATE;
 }
 
@@ -394,6 +414,8 @@ gr_window_finalize (GObject *object)
                 self->shopping_timeout_id = 0;
         }
 
+        g_list_free (self->dialogs);
+
         G_OBJECT_CLASS (gr_window_parent_class)->finalize (object);
 }
 
@@ -514,6 +536,7 @@ gr_window_class_init (GrWindowClass *klass)
         gtk_widget_class_bind_template_child (widget_class, GrWindow, header_start_stack);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, header_title_stack);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, header_end_stack);
+        gtk_widget_class_bind_template_child (widget_class, GrWindow, back_button);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, search_button);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, cooking_button);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, search_bar);
@@ -567,6 +590,8 @@ gr_window_go_back (GrWindow *window)
                 gtk_stack_set_visible_child_name (GTK_STACK (window->header_title_stack), "main");
                 gtk_stack_set_visible_child_name (GTK_STACK (window->header_end_stack), "search");
 
+                gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), _("Recipes"));
+
                 gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "recipes");
         }
         else {
@@ -643,29 +668,68 @@ file_chooser_response (GtkNativeDialog *self,
                 file = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (self));
                 do_import (window, file);
         }
+
+        gtk_native_dialog_destroy (self);
+        window->file_chooser = NULL;
+}
+
+static void
+chef_done (GrChefDialog *dialog,
+           GrChef       *chef,
+           GrWindow     *window)
+{
+        window->chef_dialog = NULL;
+        gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
+void
+gr_window_show_my_chef_information (GrWindow *window)
+{
+        GrRecipeStore *store;
+        const char *id;
+        g_autoptr(GrChef) chef = NULL;
+
+        if (window->chef_dialog)
+                return;
+
+        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+
+        id = gr_recipe_store_get_user_key (store);
+
+        if (id != NULL && id[0] != '\0')
+                chef = gr_recipe_store_get_chef (store, id);
+        else
+                chef = gr_chef_new ();
+
+        window->chef_dialog = (GtkWidget *)gr_chef_dialog_new (chef, FALSE);
+        gtk_window_set_title (GTK_WINDOW (window->chef_dialog), _("My Chef Information"));
+        g_signal_connect (window->chef_dialog, "done", G_CALLBACK (chef_done), window);
+
+        gr_window_present_dialog (window, GTK_WINDOW (window->chef_dialog));
 }
 
 void
 gr_window_load_recipe (GrWindow *window,
                        GFile    *file)
 {
-        GtkFileChooserNative *chooser;
-
         if (file) {
                 do_import (window, file);
                 return;
         }
 
-        chooser = gtk_file_chooser_native_new (_("Select a recipe file"),
-                                               GTK_WINDOW (window),
-                                               GTK_FILE_CHOOSER_ACTION_OPEN,
-                                               _("Open"),
-                                               _("Cancel"));
-        gtk_native_dialog_set_modal (GTK_NATIVE_DIALOG (chooser), TRUE);
+        if (window->file_chooser)
+                return;
 
-        g_signal_connect (chooser, "response", G_CALLBACK (file_chooser_response), window);
+        window->file_chooser = (GObject *)gtk_file_chooser_native_new (_("Select a recipe file"),
+                                                                       GTK_WINDOW (window),
+                                                                       GTK_FILE_CHOOSER_ACTION_OPEN,
+                                                                       _("Open"),
+                                                                       _("Cancel"));
+        gtk_native_dialog_set_modal (GTK_NATIVE_DIALOG (window->file_chooser), TRUE);
 
-        gtk_native_dialog_show (GTK_NATIVE_DIALOG (chooser));
+        g_signal_connect (window->file_chooser, "response", G_CALLBACK (file_chooser_response), window);
+
+        gtk_native_dialog_show (GTK_NATIVE_DIALOG (window->file_chooser));
 }
 
 void
@@ -847,3 +911,51 @@ gr_window_show_image (GrWindow *window,
                 gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "details");
         }
 }
+
+static void
+dialog_unmapped_cb (GtkWidget *dialog,
+                    GrWindow  *window)
+{
+        window->dialogs = g_list_remove (window->dialogs, dialog);
+}
+
+void
+gr_window_present_dialog (GrWindow  *window,
+                          GtkWindow *dialog)
+{
+        GtkWindow *parent;
+
+        if (window->dialogs)
+                parent = window->dialogs->data;
+        else
+                parent = GTK_WINDOW (window);
+
+        gtk_window_set_transient_for (dialog, parent);
+        gtk_window_set_modal (dialog, TRUE);
+
+        window->dialogs = g_list_prepend (window->dialogs, dialog);
+        g_signal_connect (dialog, "unmap",
+                          G_CALLBACK (dialog_unmapped_cb), window);
+
+        gtk_window_present (dialog);
+}
+
+static void
+about_response (GtkWidget *dialog,
+                int        response,
+                GrWindow  *window)
+{
+        gtk_widget_destroy (dialog);
+        window->about_dialog = NULL;
+}
+
+void
+gr_window_show_about_dialog (GrWindow *window)
+{
+        if (window->about_dialog)
+                return;
+
+        window->about_dialog = (GtkWidget *)gr_about_dialog_new ();
+        g_signal_connect (window->about_dialog, "response", G_CALLBACK (about_response), window);
+        gr_window_present_dialog (window, GTK_WINDOW (window->about_dialog));
+}
