@@ -34,9 +34,13 @@
 #include "gr-recipes-page.h"
 #include "gr-search-page.h"
 #include "gr-shopping-page.h"
+#include "gr-cooking-view.h"
+#include "gr-cooking-page.h"
 #include "gr-timer-widget.h"
+#include "gr-time-widget.h"
 #include "gr-toggle-button.h"
 #include "gr-image-viewer.h"
+#include "gr-image-page.h"
 
 
 int
@@ -45,17 +49,21 @@ main (int argc, char *argv[])
         g_autoptr (GApplication) app = NULL;
         int status;
 
+        g_type_ensure (GR_TYPE_COOKING_VIEW);
+        g_type_ensure (GR_TYPE_COOKING_PAGE);
         g_type_ensure (GR_TYPE_CUISINE_PAGE);
         g_type_ensure (GR_TYPE_CUISINES_PAGE);
         g_type_ensure (GR_TYPE_DETAILS_PAGE);
         g_type_ensure (GR_TYPE_EDIT_PAGE);
         g_type_ensure (GR_TYPE_IMAGE_VIEWER);
+        g_type_ensure (GR_TYPE_IMAGE_PAGE);
         g_type_ensure (GR_TYPE_LIST_PAGE);
         g_type_ensure (GR_TYPE_QUERY_EDITOR);
         g_type_ensure (GR_TYPE_RECIPES_PAGE);
         g_type_ensure (GR_TYPE_SEARCH_PAGE);
         g_type_ensure (GR_TYPE_SHOPPING_PAGE);
         g_type_ensure (GR_TYPE_TIMER_WIDGET);
+        g_type_ensure (GR_TYPE_TIME_WIDGET);
         g_type_ensure (GR_TYPE_TOGGLE_BUTTON);
 
         setlocale (LC_ALL, "");
